//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#ifndef YCSConnect_h
#define YCSConnect_h

#import <Foundation/Foundation.h>
#import <YounifyConnectSDK/Logging.h>
#import <YounifyConnectSDK/Tokens.h>
#import <YounifyConnectSDK/Cancellation.h>

NS_ASSUME_NONNULL_BEGIN

/// Younify Connect SDK startup options.
///
/// Set the properties of this object to configure the sdk.
/// The ``key`` and ``tokenDelegate`` properties are required.
/// The ``accessToken`` and ``refreshToken`` properties are required for linking and fetching service content for the user and can be set at a later time if unknown at startup. The tokens can be created from the backend api.
/// The ``tokenDelegate`` must be set to handle updated tokens and requests from the sdk to renew tokens from the backend api.
@interface YCSConnectOptions : NSObject

/// Required key.
@property (nonatomic, copy, nullable) NSString *key;
/// Minimum log level. Log statements below ``logLevel`` will not be sent to ``logDelegate``.
@property (nonatomic, assign) YCSLogLevel logLevel;
/// Optional log delegate for receiving log events.
@property (nonatomic, strong, nullable) id<YCSLogDelegate> logDelegate;
/// Token delegate for handling token events and renewals.
@property (nonatomic, strong, nullable) id<YCSTokenDelegate> tokenDelegate;
/// The user's access token if known at startup.
@property (nonatomic, copy, nullable) NSString *accessToken;
/// The user's refresh token if known at startup.
@property (nonatomic, copy, nullable) NSString *refreshToken;
/// Extra options.
@property (nonatomic, copy, nullable) NSDictionary<NSString *, id> *extra;

@end


/// Streaming service link.
///
/// Contains informative user properties about the linked streaming service.
@interface YCSStreamingServiceLink : NSObject

/// Service username or email.
@property (nonatomic, copy, nullable) NSString *username;
/// Selected profile name if the service has profiles.
@property (nonatomic, copy, nullable) NSString *profileName;
/// Selected profile image if the service has profiles.
@property (nonatomic, copy, nullable) NSString *profileImageUrl;
/// Whether this profile is pin protected if the service has profiles.
@property (nonatomic, assign) BOOL isProfilePINProtected;
/// Whether this link is broken or otherwise has a problem that requires management or unlinking to address.
@property (nonatomic, assign) BOOL isBroken;

@end


/// Streaming service.
///
/// Represents a streaming service.
@interface YCSStreamingService : NSObject

/// Streaming service name.
@property (nonatomic, copy) NSString *name;
/// Streaming service small thumbnail image url.
@property (nonatomic, copy, nullable) NSString *smallThumbnailUrl;
/// Streaming service large thumbnail image url.
@property (nonatomic, copy, nullable) NSString *largeThumbnailUrl;
/// Streaming service overlay thumbnail image url.
@property (nonatomic, copy, nullable) NSString *overlayThumbnailUrl;
/// Whether this streaming service is currently available or is undergoing maintenance.
@property (nonatomic, assign) BOOL isAvailable;
/// Streaming service link info if this service was linked by the user.
@property (nonatomic, strong, nullable) YCSStreamingServiceLink *link;

@end


/// Streaming service content type.
///
/// The type of content the ``YCSStreamingContent`` object represents.
typedef NS_ENUM(NSInteger, YCSStreamingContentType)
{
    /// Invalid type.
    YCSStreamingContentTypeInvalid = -1,
    /// Folder.
    YCSStreamingContentTypeFolder,
    /// Video.
    YCSStreamingContentTypeVideo
};


/// Streaming service content.
///
/// Represents a content item from a streaming service.
@interface YCSStreamingContent : NSObject

/// Content type.
@property (nonatomic, assign) YCSStreamingContentType type;
/// Name in the path hierarchy derived from ``season``, ``episode`` and ``title``.
@property (nonatomic, copy) NSString *name;
/// Title.
@property (nonatomic, copy) NSString *title;
/// Overview, synopsis or description.
@property (nonatomic, copy, nullable) NSString *overview;
/// Duration in milliseconds.
@property (nonatomic, assign) long long duration;
/// Air date.
@property (nonatomic, copy, nullable) NSDate *airDate;
/// Small thumbnail image url.
@property (nonatomic, copy, nullable) NSString *smallThumbnailUrl;
/// Large thumbnail image url.
@property (nonatomic, copy, nullable) NSString *largeThumbnailUrl;
/// Streaming service specific content id for this item.
@property (nonatomic, copy, nullable) NSString *itemID;
/// Series title.
@property (nonatomic, copy, nullable) NSString *series;
/// Series season.
@property (nonatomic, copy, nullable) NSString *season;
/// Series episode.
@property (nonatomic, copy, nullable) NSString *episode;
/// Release year.
@property (nonatomic, copy, nullable) NSString *releaseYear;
/// Content rating.
@property (nonatomic, copy, nullable) NSString *contentRating;
/// Streaming service specific watch now url.
@property (nonatomic, copy, nullable) NSString *watchNowUrl;
/// TMDB id for this content if available.
@property (nonatomic, copy, nullable) NSString *tmdbID;
/// TMDB id for this content's series if available.
@property (nonatomic, copy, nullable) NSString *tmdbSeriesID;
/// Path to this item as a sequence of parent ``name`` properties.
@property (nonatomic, copy) NSArray<NSString *> *path;

@end


/// Streaming service content details.
///
/// Holds additional content item details for a ``YCSStreamingContent`` object.
@interface YCSStreamingContentDetails : NSObject

/// Estimated duration in milliseconds of this item including advertisements.
@property (nonatomic, assign) long long estimatedDuration;

@end


/// Streaming service content category.
///
/// Represents a streaming service category for a user.
@interface YCSStreamingCategory : NSObject

/// Category name.
@property (nonatomic, copy) NSString *name;

@end


/// Well-known streaming service content categories.
///
/// Holds a list of well-known streaming service categories.
@interface YCSStreamingCategories : NSObject

/// Your Watchlist.
+ (YCSStreamingCategory *)yourWatchlist;
/// Continue Watching.
+ (YCSStreamingCategory *)continueWatching;
/// Recommended for You.
+ (YCSStreamingCategory *)recommendedForYou;
/// Trending &amp; Popular.
+ (YCSStreamingCategory *)trendingAndPopular;
/// Critically Acclaimed.
+ (YCSStreamingCategory *)criticallyAcclaimed;

@end


/// Fetch content category service result.
///
/// Holds the results of a fetch content request for a given streaming service.
@interface YCSFetchContentCategoryServiceResult : NSObject

/// Streaming service the content items belong to.
@property (nonatomic, strong) YCSStreamingService *service;
/// Content items for `service`.
@property (nonatomic, strong) NSArray<YCSStreamingContent *> *content;

@end


/// Fetch content category result.
///
/// Holds the results of a fetch content request for a category and each streaming service.
@interface YCSFetchContentCategoryResult : NSObject

/// Streaming category the results apply to.
@property (nonatomic, strong) YCSStreamingCategory *category;
/// Service results for the specified category.
@property (nonatomic, strong) NSArray<YCSFetchContentCategoryServiceResult *> *services;

@end


/// Search content service result.
///
/// Holds the results of a search content request for a given streaming service.
@interface YCSSearchContentServiceResult : NSObject

/// Streaming service the content items belong to.
@property (nonatomic, strong) YCSStreamingService *service;
/// Content items for `service`.
@property (nonatomic, strong) NSArray<YCSStreamingContent *> *content;

@end


/// Younify Connect SDK main object.
///
/// Use the ``shared`` singleton instance to interact with the sdk. Before the sdk can be meaningfully used ``configure:`` must be called.
@interface YCSConnect : NSObject

/// The shared singleton instance.
+ (instancetype)shared;

/// Configures the sdk with the specified options.
/// - Parameter options: The configuration options.
///
/// Before using the sdk it must be configured by using this method with a valid ``YCSConnectOptions`` instance. It is safe to call this method repeatedly to change the options dynamically at runtime if desired.
- (void)configure:(YCSConnectOptions *)options;

/// Sets the user's tokens.
/// - Parameters:
///   - accessToken: The user's access token.
///   - refreshToken: The user's refresh token.
///
/// Set the user's access and refresh tokens that are required for accessing streaming services. User tokens can be obtained by creating a user object from the backend api.
- (void)setUserTokens:(nullable NSString *)accessToken refreshToken:(nullable NSString *)refreshToken;

/// Requests the user to consent to the privacy policy if it has not yet been accepted.
/// - Parameter completionHandler: The completion handler to call when the consent process is complete.
///
/// Opens the privacy policy consent flow if the privacy policy has not yet been accepted or completes immediately with TRUE if it has.
- (void)requestUserConsent:(void (^)(BOOL consented, NSError * _Nullable error))completionHandler;

/// Link a streaming service.
/// - Parameters:
///   - service: The service object to link.
///   - completionHandler: The completion handler to call when the link service request is complete.
///
/// Opens the service linking flow which involves a web view login for the specified service followed by profile selection and optional pin collection if supported. `completionHandler` will be called with the link result.
/// If linking is successful, the success flag will be TRUE and error nil. If linking was canceled then the success flag will be FALSE and error nil. If linking fails the success flag will be FALSE and error non-nil.
- (void)linkService:(YCSStreamingService *)service completionHandler:(void (^)(YCSStreamingService *service, BOOL success, NSError * _Nullable error))completionHandler;

/// Manage a linked service.
/// - Parameters:
///   - service: The service object to manage.
///   - completionHandler: The completion handler to call when the manage service request is complete.
///
/// Opens the linked service management flow which allows the user to change their profile selection and pin if supported. `completionHandler` will be called with the link management result.
- (void)manageLinkedService:(YCSStreamingService *)service completionHandler:(void (^)(YCSStreamingService *service, NSError * _Nullable error))completionHandler;

/// Unlink a linked service.
/// - Parameters:
///   - service: The service object to unlink.
///   - completionHandler: The completion handler to call when the unlink service request is complete.
///
/// Unlinks a linked service and will call `completionHandler` with the result.
- (void)unlinkService:(YCSStreamingService *)service completionHandler:(void (^)(YCSStreamingService *service, NSError * _Nullable error))completionHandler;

/// Fetch all services.
/// - Parameter completionHandler: The completion handler to call when the fetch services request is complete.
///
/// Retrieves a list of all services and will call `completionHandler` with the result.
- (void)fetchServices:(void (^)(NSArray<YCSStreamingService *> *services, NSError * _Nullable error))completionHandler;

/// Fetch linked services.
/// - Parameter completionHandler: The completion handler to call when the fetch linked services request is complete.
///
/// Retrieves a list of all the user's linked services and will call `completionHandler` with the result.
- (void)fetchLinkedServices:(void (^)(NSArray<YCSStreamingService *> *services, NSError * _Nullable error))completionHandler;

/// Fetch broken services.
/// - Parameter completionHandler: The completion handler to call when the fetch broken services request is complete.
///
/// Retrieves a list of all the user's broken services and will call `completionHandler` with the result.
- (void)fetchBrokenServices:(void (^)(NSArray<YCSStreamingService *> *services, NSError * _Nullable error))completionHandler;

/// Fetch all streaming categories.
/// - Parameter completionHandler: The completion handler to call when the fetch categories request is complete.
///
/// Retrieves a list of all streaming categories and will call `completionHandler` with the result.
- (void)fetchCategories:(void (^)(NSArray<YCSStreamingCategory *> *categories, NSError * _Nullable error))completionHandler;

/// Fetch streaming category contents.
/// - Parameters:
///   - categories: The list of categories to fetch content for.
///   - services: The list of streaming services to fetch content for.
///   - serviceErrorHandler: The optional error handler to call when a problem occurs accessing a service.
///   - categoryServiceCompletionHandler: The optional completion handler to call when retrieving content for a single category and service.
///   - allCompletionHandler: The optional completion handler to call when all content has been retrieved.
///   - cancellationTicket: The ticket to manually cancel the operation.
///
/// Retrieves the content for the `categories` and `services` specified. `serviceErrorHandler` will be called when a service error occurs. `categoryServiceCompletionHandler` will be called with the results for a single category-service pair while `allCompletionHandler` will be called with all of the results.
- (void)fetchContent:(NSArray<YCSStreamingCategory *> *)categories
        services:(NSArray<YCSStreamingService *> *)services
        serviceErrorHandler:(nullable void (^)(YCSStreamingService *service, NSError * _Nullable error))serviceErrorHandler
        categoryServiceCompletionHandler:(nullable void (^)(YCSStreamingCategory *category, YCSStreamingService *service, NSArray<YCSStreamingContent *> *content, NSError * _Nullable error))categoryServiceCompletionHandler
        allCompletionHandler:(nullable void (^)(NSArray<YCSFetchContentCategoryResult *> *result, NSError * _Nullable error))allCompletionHandler
        cancellationTicket:(nullable YCSCancellationTicket *)cancellationTicket;

/// Fetch content children.
/// - Parameters:
///   - content: The parent content item to fetch children for.
///   - completionHandler: The completion handler to call when the fetch children request is complete.
///
/// Retrieves the children of the given parent `content` object and will call `completionHandler` with the results. The `content` object must be a folder to have children.
///
/// Examples include using a `content` that represents a series where the children would be its seasons, or a `content` that represents a season where the children would be its episodes.
- (void)fetchContentChildren:(YCSStreamingContent *)content completionHandler:(void (^)(YCSStreamingContent *content, NSArray<YCSStreamingContent *> *children, NSError * _Nullable error))completionHandler;

/// Fetch content details.
/// - Parameters:
///   - content: The content item to retrieve details for.
///   - completionHandler: The completion handler to call when the fetch content details request is complete.
///
/// Retrieves `content` details and will call `completionHandler` with the results.
///
/// Details or additional metadata properties currently include estimated durations.
- (void)fetchContentDetails:(YCSStreamingContent *)content completionHandler:(void (^)(YCSStreamingContent *content, YCSStreamingContentDetails * _Nullable details, NSError * _Nullable error))completionHandler;

/// Search streaming services.
/// - Parameters:
///   - services: The list of streaming services to search content for.
///   - searchPhrase: The search phrase or terms.
///   - serviceCompletionHandler: The optional completion handler to call when retrieving search results for a single service.
///   - allCompletionHandler: The optional completion handler to call when all search results have been retrieved.
///   - cancellationTicket: The ticket to manually cancel the operation.
///
/// Searches the given `services` with the specified `searchPhrase`. `serviceCompletionHandler` will be called with the results for a single service while `allCompletionHandler` will be called with all of the results.
- (void)    searchContent:(NSArray<YCSStreamingService *> *)services
             searchPhrase:(NSString *)searchPhrase
 serviceCompletionHandler:(nullable void (^)(YCSStreamingService *service, NSArray<YCSStreamingContent *> *content, NSError * _Nullable error))categoryServiceCompletionHandler
     allCompletionHandler:(nullable void (^)(NSArray<YCSSearchContentServiceResult *> *content, NSError * _Nullable error))allCompletionHandler
       cancellationTicket:(nullable YCSCancellationTicket *)cancellationTicket;

/// Fetch an image.
/// - Parameters:
///   - url: The image url to fetch.
///   - completionHandler: The completion handler to call when the fetch image request is complete.
///
/// Retrieves the image from the specified image `url` and will call `completionHandler` with the image data result.
///
/// Directly fetching image urls in any other way can lead to missing data or 404s depending on the streaming service.
- (void)fetchImage:(NSString *)url completionHandler:(void (^)(NSData * _Nullable image, NSError * _Nullable error))completionHandler;

@end

NS_ASSUME_NONNULL_END

#endif /* YCSConnect_h */
