//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#ifndef YCSTokens_h
#define YCSTokens_h

NS_ASSUME_NONNULL_BEGIN

/// Token delegate.
///
/// Implement this delegate to receive token events from the sdk and to handle backend token renewals. The instance can be set in ``YCSConnectOptions``.
@protocol YCSTokenDelegate

/// Called when an access token is missing or expired that must be renewed via the backend.
/// - Parameters:
///   - expiredAccessToken: The user's expired access token.
///   - refreshToken: The user's refresh token for the expired access token.
///   - completionHandler: The completion handler to be called with the new access and refresh tokens on success or nil if it fails.
///
/// This method will be called when an access token is missing or expired and needs renewing via the backend. The SDK will first attempt automatic renewal of tokens via the frontend, but if this fails because
/// the refresh token is missing, revoked or of some other error, a backend renewal will be required. Backend renewals require proxying the request through your own backend api to Younify using your private api key.
/// The `completionHandler` must be called with the new tokens or nil if an error occurs. As a convenience, `renewedTokens` will be called afterward on success to persist the new tokens in device storage.
- (void)renewTokens:(nullable NSString *)expiredAccessToken refreshToken:(nullable NSString *)refreshToken completionHandler:(void (^)(NSString * _Nullable newAccessToken, NSString * _Nullable newRefreshToken))completionHandler;

/// Called when an access token and refresh token are renewed and need persisted.
/// - Parameters:
///   - newAccessToken: The user's newly acquired access token.
///   - newRefreshToken: The user's newly acquired refresh token.
///
/// This method will be called when an access token and refresh token have been renewed and need persisted. The listener should persisted the new tokens in device local storage in a non-cloud-synchronized manner.
/// If the tokens are accidentally shared between multiple devices, frontend renewal on one device may cause the other devices to be logged out triggering a backend renewal request via `renewTokens`.
- (void)renewedTokens:(NSString *)newAccessToken refreshToken:(NSString *)newRefreshToken;

@end

NS_ASSUME_NONNULL_END

#endif /* YCSTokens_h */
