//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#ifndef YCSErrors_h
#define YCSErrors_h

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/// Error domain used for errors reported by the SDK.
extern NSString *const YCSErrorDomain;

/// NSError userInfo dictionary key containing an NSObject of the partial result if available.
extern NSString *const YCSErrorUserInfoPartialResultKey;

/// NSError userInfo dictionary key containing an NSNumber of the HTTP statusCode for failed network requests.
extern NSString *const YCSErrorUserInfoStatusCodeKey;

/// NSError userInfo dictionary key containing an NSString of the parameter name that caused the invalid argument error.
extern NSString *const YCSErrorUserInfoParamNameKey;

/// NSError userInfo dictionary key containing an NSString of the streaming service specific error message when linking fails.
extern NSString *const YCSErrorUserInfoServiceMessageKey;


/// Error codes reported by the SDK.
typedef NS_ENUM(NSInteger, YCSError)
{
    /// An unknown error occurred.
    YCSErrorUnknown = -10000,

    /// The current version of the SDK is no longer supported.
    YCSErrorSDKVersionNotSupported = -10001,

    /// The system is currently down for maintenance.
    YCSErrorSystemMaintenance = -10002,

    /// A network error occurred. The user is either offline or the server returned an invalid response. The specific http status code is available in the statusCode property of the userInfo dictionary.
    YCSErrorNetworkRequestFailed = -10003,

    /// The requested operation has been canceled.
    YCSErrorOperationCanceled = -10004,

    /// An invalid argument was encountered.
    YCSErrorInvalidArgument = -10005,


    /// The service fetch operation failed.
    YCSErrorServiceFetchingFailed = -20000,

    /// The service linking operation failed.
    YCSErrorServiceLinkingFailed = -20001,

    /// The service is not supported by the SDK.
    YCSErrorServiceUnsupported = -20002,

    /// The service is temporarily unavailable.
    YCSErrorServiceTemporarilyUnavailable = -20003,

    /// The service login failed. The user should verify their login information.
    YCSErrorServiceLoginFailed = -20004,

    /// The service profile was not found. The user should select another profile.
    YCSErrorServiceProfileNotFound = -20005,

    /// The service profile pin was not correct. The user should enter their pin again.
    YCSErrorServiceProfilePinMismatch = -20006,

    /// This service content may no longer be available.
    YCSErrorServiceContentNotFound = -20007,


    /// The view context is invalid.
    YCSErrorInvalidViewContext = -30000,


    /// The user tried to perform an action that requires privacy policy consent.
    YCSErrorUserConsentRequired = -40000
};

/// Errors utility class.
@interface YCSErrors : NSObject

/// Determines if the given error was generated by the sdk.
/// - Parameters:
///   - error: Any error.
+ (BOOL)isError:(NSError *)error;

/// Determines if the given error was generated by the sdk and matches the specified code.
/// - Parameters:
///   - error: Any error.
///   - code: The code to check against.
+ (BOOL)isError:(NSError *)error code:(YCSError)code;

/// Creates an error for the given code.
/// - Parameters:
///   - code: The code to create an error for.
///   - userInfo: Optional dictionary of ``NSErrorUserInfoKey`` and ``NSObject`` pairs to include in the error. Valid keys additionally include the ``YCSErrorUserInfo*Key`` constants.
///
/// Creates an ``NSError`` for the given error ``code``.
/// The optional ``userInfo`` dictionary can include additional data for the error. Valid keys include the constants matching the pattern ``YCSErrorUserInfo*Key``.
/// If ``userInfo`` is nil or does not contain ``NSLocalizedDescriptionKey`` a default description corresponding to ``code`` will be supplied.
+ (NSError *)createError:(YCSError)code userInfo:(NSDictionary<NSErrorUserInfoKey, id> * _Nullable)userInfo;

@end

NS_ASSUME_NONNULL_END

#endif /* YCSErrors_h */
